-- LuaTools需要PROJECT和VERSION这两个信息
PROJECT = "gnsstest"
VERSION = "1.0.1"

--[[
本demo需要很多流量!!!
注意: 室内无信号!! 无法定位!!!
]]

-- sys库是标配
local sys = require("sys")
require("sysplus")

-- Air780E的AT固件默认会为开机键防抖, 导致部分用户刷机很麻烦
if rtos.bsp() == "EC618" and pm and pm.PWK_MODE then
    pm.power(pm.PWK_MODE, false)
end

local pm_start = {1, 2, 3, pm.IDLE} -- 分别对应响应优先，均衡模式，PSM+模式，和正常运行不休眠模式
local lbs_lat, lbs_lng, lbs_time_table = nil, nil, nil -- 基站定位的经纬度以及基站定位服务器的时间

local pm_ready = pm_start[3] -- 真正进入的休眠模式

local send_mqtt_data_ready = nil

local send_timer = 1 -- GPS数据上报时间（单位：分钟）
local send_timer = send_timer * 60 * 1000
-- GPS芯片所在串口
local gps_uart_id = 2

local mqttc = nil

libgnss.clear() -- 清空数据,兼初始化

log.info("open GPS HH")
log.info("GPS_PM_ OPEN 21", pm.power(pm.GPS, true))
mobile.flymode(1, false)

uart.setup(gps_uart_id, 115200)

local reason, slp_state = pm.lastReson()
log.info("唤醒方式", pm.lastReson())

if reason > 0 then
    log.info("Has woken up from deep sleep")
    -- pm.power(pm.WORK_MODE,pm_ready)
end

-- 星历处理函数
function exec_agnss()
    -- 如果是热启动，不用重写星历
    if http and reason == 0 then
        -- AGNSS 已调通
        while 1 do
            local code, headers, body = http.request("GET",
                                                     "http://download.openluat.com/9501-xingli/HXXT_GPS_BDS_AGNSS_DATA.dat")
                                            .wait()

            log.info("gnss", "AGNSS", code, body and #body or 0)
            if code == 200 and body and #body > 1024 then
                for offset = 1, #body, 512 do
                    log.info("gnss", "AGNSS", "write >>>",
                             #body:sub(offset, offset + 511))
                    uart.write(gps_uart_id, body:sub(offset, offset + 511))

                    sys.wait(100) -- 等100ms反而更成功
                end

                io.writeFile("/6228.bin", body)

                log.info("Ephemeris_file_was_written_successfully")
                break
            end
            sys.wait(60 * 1000)
        end
    end
    sys.wait(20)
    -- "$AIDTIME,year,month,day,hour,minute,second,millisecond"
    local date = os.date("!*t")
    if date.year > 2022 then
        local str = string.format("$AIDTIME,%d,%d,%d,%d,%d,%d,000",
                                  date["year"], date["month"], date["day"],
                                  date["hour"], date["min"], date["sec"])
        log.info("gnss", str)
        uart.write(gps_uart_id, str .. "\r\n")
        sys.wait(20)
    end
    -- 读取之前的位置信息
    local gnssloc = io.readFile("/gnssloc")
    if gnssloc then
        str = "$AIDPOS," .. gnssloc
        log.info("POS", str)
        uart.write(gps_uart_id, str .. "\r\n")
        str = nil
        gnssloc = nil
    else
        -- TODO 发起基站定位
        uart.write(gps_uart_id, "$AIDPOS,3432.70,N,10885.25,E,1.0\r\n")
    end
end

sys.taskInit(function()
    -- Air780EG工程样品的GPS的默认波特率是9600, 量产版是115200,以下是临时代码
    log.info("GPS", "start")
    log.info("OPEN GPS HH")
    -- pm.power(pm.GPS, true)
    log.info("GPS_PM_ OPEN 111", pm.power(pm.GPS, true))

    -- 绑定uart,底层自动处理GNSS数据
    -- 第二个参数是转发到虚拟UART, 方便上位机分析
    libgnss.bind(gps_uart_id, uart.VUART_0)

    -- 发送给MQTT的实际数据
    libgnss.on("raw", function(data)
        -- 默认不上报, 需要的话自行打开
        data = data:split("\r\n")
        -- 底层没有数据上来就不发给mqtt服务器
        if data == nil then return end
        -- --  不需要发数据给mqtt的场景就不用发
        if send_mqtt_data_ready == nil then
            log.info(
                "There is no need to send gnss data to the mqtt server for now")
            return
        end
        sys.taskInit(function()
            for k, v in pairs(data) do
                if v and v:startsWith("$GNRMC") then
                    sys.publish("mqtt_pub",
                                "/gnss/" .. mobile.imei() .. "/up/nmea", v, 0)
                    log.info("The GNSS data is sent once to the MQTT server")
                    log.info(
                        "After three minutes, turn off the GPS and go to sleep")
                        --首次冷启动定位成功以后建议延迟两分半，主要目的是为了搜到完整的星历给GNSS模块
                  if reason==0 then
                    sys.wait( 150 * 1000)
                  end
                    -- 只发给服务器一次当前点的数据，发送完就不发了，下一次发送是定时器唤醒
                    -- 进休眠前主动关闭GPS电源试试

                    log.info("CLOSE GPS HH")
                    log.info("GPS_PM_ CLSOE 141", pm.power(pm.GPS, false))
                    -- 进psm+前先进飞行
                    mobile.flymode(1, true)
                    pm.power(pm.WORK_MODE, pm_ready)
                    send_mqtt_data_ready = nil
                    -- 发完就进定时器唤醒GPS电源
                    pm.dtimerStart(3, send_timer)

                    gpio.close(35)
                    gpio.setup(20, function()
                        log.info("gpio")
                    end, gpio.PULLUP, gpio.FALLING)
                end
            end
        end)

    end)
    sys.wait(200) -- GPNSS芯片启动需要时间
    -- 调试日志,可选
    libgnss.debug(true)
    -- 显示串口配置
    -- uart.write(gps_uart_id, "$CFGPRT,1\r\n")
    -- sys.wait(20)
    -- 增加显示的语句
    uart.write(gps_uart_id, "$CFGMSG,0,1,1\r\n") -- GLL
    sys.wait(20)
    uart.write(gps_uart_id, "$CFGMSG,0,5,1\r\n") -- VTG
    sys.wait(20)
    uart.write(gps_uart_id, "$CFGMSG,0,6,1\r\n") -- ZDA
    sys.wait(20)
    -- 定位成功后,使用GNSS时间设置RTC, 暂不可用
    -- libgnss.rtcAuto(true)
    exec_agnss()
end)

-- 订阅GNSS状态编码
sys.subscribe("GNSS_STATE", function(event, ticks)
    -- event取值有
    -- FIXED 定位成功
    -- LOSE  定位丢失
    -- ticks是事件发生的时间,一般可以忽略
    log.info("gnss", "state", event, ticks)
    if event == "FIXED" then
        local locStr = libgnss.locStr()

        log.info("gnss_HH", "fixed", locStr)

        send_mqtt_data_ready = true

        if locStr then io.writeFile("/gnssloc", locStr) end
    end
end)

-- mqtt 上传任务
sys.taskInit(function()
    sys.waitUntil("IP_READY", 15000)

    mqttc = mqtt.create(nil, "lbsmqtt.airm2m.com", 1886) -- mqtt客户端创建
    mqttc:auth(mobile.imei(), mobile.imei(), mobile.muid()) -- mqtt三元组配置

    log.info("mqtt", mobile.imei(), mobile.imei(), mobile.muid())
    mqttc:keepalive(30) -- 默认值240s
    mqttc:autoreconn(true, 3000) -- 自动重连机制

    mqttc:on(function(mqtt_client, event, data, payload) -- mqtt回调注册
        -- 用户自定义代码，按event处理
        -- log.info("mqtt", "event", event, mqtt_client, data, payload)
        if event == "conack" then -- mqtt成功完成鉴权后的消息
            sys.publish("mqtt_conack") -- 小写字母的topic均为自定义topic
            -- 订阅不是必须的，但一般会有
            mqtt_client:subscribe("/gnss/" .. mobile.imei() .. "/down/#")
        elseif event == "recv" then -- 服务器下发的数据
            -- 这里继续加自定义的业务处理逻辑
            log.info("mqtt", "downlink", "topic", data, "payload", payload)
        elseif event == "sent" then -- publish成功后的事件
            log.info("mqtt", "sent", "pkgid", data)
        end
    end)

    -- 发起连接之后，mqtt库会自动维护链接，若连接断开，默认会自动重连
    mqttc:connect()
    sys.waitUntil("mqtt_conack")
    log.info("mqtt conack ok")
    -- sys.timerStart(upload_stat, 1000) -- 一秒后主动上传一次
    while true do
        -- 业务演示。等待其他task发过来的待上报数据
        -- 这里的mqtt_pub字符串是自定义的，与mqtt库没有直接联系
        -- 若不需要异步关闭mqtt链接，while内的代码可以替换成sys.wait(30000)
        local ret, topic, data, qos = sys.waitUntil("mqtt_pub", 500)
        if ret then
            if topic == "close" then break end
            log.info("mqtt", "publish", "topic", topic)
            mqttc:publish(topic, data, qos)
        end
    end
    mqttc:close()
    mqttc = nil
end)

sys.taskInit(function()
    while 1 do
        sys.wait(3600 * 1000) -- 一小时检查一次星历
        local fixed, time_fixed = libgnss.isFix()
        if not fixed then exec_agnss() end
    end
end)

sys.run()